#!/usr/bin/env python3

# https://pythonprogramming.altervista.org/simple-labytinth-in-pygame/

import os
import sys
import random
import pygame

# Holds the level layout in a list of strings.
level = [
    "WWWWWWWWWWWWWWWWWWWW",
    "W     W            W",
    "W S   W            W",
    "W            W   E W",
    "W            W     W",
    "W            W     W",
    "WWWWWWWWWWWWWWWWWWWW",
]


step = 32 # field size
delay = 1000 # milliseconds between movements

# Class for the orange dude
class Player(object):

    def __init__(self, board, img_path = None):
        self.rect = pygame.Rect(2*step, 2*step, step, step)
        self.board = board
        if img_path is None:
            self.img = None
        else:
            self.img = pygame.image.load(img_path)
    def move(self, dx, dy):

        # Move each axis separately. Note that this checks for collisions both times.
        if dx != 0:
            self.move_single_axis((dx/abs(dx))*step, 0)
        if dy != 0:
            self.move_single_axis(0, (dy/abs(dy))*step)

        self.board.clock.tick(500)

    def MoveUp(self):
        self.MovementProcess(0, -step)
    def MoveDown(self):
        self.MovementProcess(0, +step)
    def MoveLeft(self):
        self.MovementProcess(-step, 0)
    def MoveRight(self):
        self.MovementProcess(+step, 0)
    def Wait(self):
        self.MovementProcess(0, 0)


    def MoveTo(self, pos):
        self.rect.x = pos[0]
        self.rect.y = pos[1]
        self.move_single_axis(0, 0)

    def MovementProcess(self, *args):
        for e in pygame.event.get():
            if e.type == pygame.QUIT:
                running = False
            if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
                running = False

        # quit with the "q" key
        key = pygame.key.get_pressed()
        if key[pygame.K_q]:
            pygame.quit()
            sys.exit()

        self.move_single_axis(*args)
        self.board.DrawScene()
        pygame.time.wait(delay)


    def move_single_axis(self, dx, dy):

        # Move the rect
        self.rect.x += dx
        self.rect.y += dy

        # If you collide with a wall, move out based on velocity
        for wall in self.board.walls:
            if self.rect.colliderect(wall.rect):
                if dx > 0: # Moving right; Hit the left side of the wall
                    self.rect.right = wall.rect.left
                if dx < 0: # Moving left; Hit the right side of the wall
                    self.rect.left = wall.rect.right
                if dy > 0: # Moving down; Hit the top side of the wall
                    self.rect.bottom = wall.rect.top
                if dy < 0: # Moving up; Hit the bottom side of the wall
                    self.rect.top = wall.rect.bottom

# Nice class to hold a wall rect
class Wall(object):

    def __init__(self, board, pos):
        board.walls.append(self)
        self.rect = pygame.Rect(pos[0], pos[1], step, step)

class TheGame(object):

    def __init__(self, level, img_path = None, snoep_img = None):
        # Initialise pygame
        os.environ["SDL_VIDEO_CENTERED"] = "1"
        pygame.init()

        # Set up the display
        pygame.display.set_caption("Get to the red square!")
        resolution = (len(level[0])*step, len(level)*step)
        self.screen = pygame.display.set_mode(resolution)

        self.clock = pygame.time.Clock()
        self.walls = [] # List to hold the walls

        # win/fail states
        self.win = False
        self.fail = False

        # set player
        self.player = Player(board = self, img_path = img_path) # Create the player

        # prepare the level
        self.LevelSetup(level, snoep_img = snoep_img)


    def LevelSetup(self, level, snoep_img = None):

        # Parse the level string above. W = wall, E = exit
        x = y = 0
        for row in level:
            for col in row:
                if col == "W":
                    Wall(self, (x, y))
                if col == "S":
                    self.player.MoveTo((x,y))
                if col == "E":
                    self.end_rect = pygame.Rect(x, y, step, step)
                    if snoep_img is None:
                        self.end_img = None
                    else:
                        self.end_img = pygame.image.load(snoep_img)
                x += step
            y += step
            x = 0


    def RunFunction(self, Function):
        self.clock.tick(16)
        self.DrawScene()

        pygame.display.update()
        pygame.time.wait(delay)

        Function()

        if self.player.rect.colliderect(self.end_rect):
            self.win = True
            print ("you won!")

        if not self.win:
            self.fail = True


        self.DrawScene()
        pygame.time.wait(5*delay)
        pygame.quit()
        sys.exit()


    def Run(self, commands = None):
        running = True
        while running:

            self.clock.tick(16)

            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
                    running = False

            # Move the player if an arrow key is pressed
            key = pygame.key.get_pressed()
            if key[pygame.K_LEFT]:
                self.player.move(-1, 0)
            if key[pygame.K_RIGHT]:
                self.player.move(1, 0)
            if key[pygame.K_UP]:
                self.player.move(0, -1)
            if key[pygame.K_DOWN]:
                self.player.move(0, 1)


            # quit with the "q" key
            if key[pygame.K_q]:
                pygame.quit()
                sys.exit()

            self.DrawScene()
            self.clock.tick(360)

            if self.player.rect.colliderect(self.end_rect):
                self.win = True
                print("you won!")
                self.DrawScene()
                pygame.time.wait(3*delay)
                pygame.quit()
                sys.exit()
        pygame.quit()


    def DrawScene(self):
        # Draw the scene
        self.screen.fill((0, 0, 0))
        for wall in self.walls:
            if self.win:
                color = (0, 200, 0)
            elif self.fail:
                color = (255, 80, 80)
            else:
                color = (255, 255, 255)
            pygame.draw.rect(self.screen, color, wall.rect)
        pygame.draw.rect(self.screen, (255, 0, 0), self.end_rect)
        pygame.draw.rect(self.screen, (255, 200, 0), self.player.rect)
        if self.end_img is not None:
            self.screen.blit(self.end_img,(self.end_rect.x,self.end_rect.y))
        if self.player.img is not None:
            self.screen.blit(self.player.img,(self.player.rect.x,self.player.rect.y))



        pygame.display.update()


if __name__ == "__main__":
    game = TheGame(level, img_path = 'pompoen.jpg', snoep_img = 'snoep.jpg')
    player = game.player

    game.Run()

    #
    def MyTestFunction():
        player.MoveUp()
        player.MoveRight()

    game.RunFunction(MyTestFunction)
